# -*- coding: utf-8 -*-

# This file is part of EUhu.
#
# EUhu is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# EUhu is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License


from collections import namedtuple

Mappingresult = namedtuple('Mappingresult', 'esf errors')

rules = []


def rule(func):
    """registers decorated rules"""
    rules.append(func)
    return func


def check_fields(fields, bmas, esf):
    '''
    Helper function for rules

    This function simply checks that all fields needed for the computation of
    the rule are present in the data dictionaries. If one is missing a KeyError
    is raised and the whole rule fails.

    While some rules could be evaluated without all arguments (e.g. A or B or C
    with only A being available), the strict behaviour was implemented by the
    team as to not get a false sense of security.
    '''
    for field in fields:
        if field not in bmas and field not in esf:
            raise KeyError(field)


def to_esf(bmas, esf=None):
    """Map BMAS to ESF fields

    Maps incoming bmas dictionary to incoming esf dictionary with every
    decorated @rule
    """
    esf = esf or {}
    return map_bmas_to_esf(bmas, esf, *rules)


def map_bmas_to_esf(bmas, esf, *mapping_rules):
    """Maps incoming bmas to incoming esf with a defined ruleset.
       The names of the rules passed in are used as the target key for esf
       dictionary. If the mapping fails, the name of the failing rule is
       appended to map_errors. The Mappingresult is a named tuple, containing
       esf and map_errors
    """
    map_errors = []
    for rule in mapping_rules:

        # Convert method name to rule.
        #
        # - Names are upper-cased
        # - Underscores are transformed to dot
        rule_name = rule.__code__.co_name.upper().replace('_', '.')
        try:
            if rule(bmas, esf):
                esf[rule_name] = True
            else:
                esf[rule_name] = False
        except KeyError:
            map_errors.append(rule_name)
    return Mappingresult(esf, map_errors)


@rule
def co02(bmas, esf):
    check_fields(['12.3', '12.4', '12.2', '8'], bmas, esf)

    # Grouping tests for 12.3 and 12.4, since it makes more logical sense.
    return (bmas['12.3'] and not bmas['12.4']) \
        or ((bmas['12.2'] or bmas['12.3']) and bmas['8'])


@rule
def co01(bmas, esf):
    # changed because of 160803_Mapping_TN_Fragebogen_DATES-II.xlsx
    # from a logical point of view the implementation of the rule is ugly.
    # The rule should be: 12.1 or 12.2 or 12.3 or CO02 or (11.2 and 14.2).
    check_fields(['11.2', '11.3', '12.1', '12.2', '12.3', '14.2', 'CO02'],
                 bmas, esf)
    return (((bmas['12.1'] or bmas['12.2'] or bmas['12.3'])
             and not bmas['14.2'])
            or (bmas['14.2'] and bmas['11.2'])
            or esf['CO02']
            or ((bmas['12.1'] or bmas['12.2'] or bmas['12.3'])
                and bmas['14.2'])
            or (bmas['11.2'] and bmas['11.3'] and bmas['14.2']))


@rule
def co03(bmas, esf):
    check_fields(['14.9',
                  '14.4',
                  '14.6',
                  '14.7',
                  '13',
                  '12.1',
                  '12.2',
                  '12.3',
                  '14.1',
                  '14.2',
                  '14.3',
                  '14.5'], bmas, esf)

    return ((bmas['14.9'] or bmas['14.4'] or bmas['14.6'] or bmas['14.7'])
            or
            (bmas['13']
             and
             (not bmas['12.1'] and not bmas['12.2'] and not bmas['12.3']
              and not bmas['14.1'] and not bmas['14.2'] and not bmas['14.3']
              and not bmas['14.5'])
             ))


@rule
def co04(bmas, esf):
    """
    Rule for CO04 indicator

    Mapping was changed, see waskiq/issue783 for more details
    """
    check_fields(['CO03', '14.4', '14.6', '14.7', '14.8'], bmas, esf)
    return esf['CO03'] and not (
        bmas['14.4'] or bmas['14.6'] or bmas['14.7'] or bmas['14.8'])


@rule
def co05(bmas, esf):
    """
    special case: "Elternzeit"
    https://support.intevation.de/waskiq/issue647
    """
    # keep the special case and update the rule
    check_fields(['11.3','14.1', '14.2', '14.3', '14.5', '14.9',
                  'CO01', 'CO03'], bmas, esf)
    if bmas['14.1'] and bmas['14.9']:
        return False
    return (bmas['14.1']
            or (bmas['14.2'] and not (esf['CO01'] or esf['CO03']))
            or bmas['14.3'] or bmas['14.5']
            or ((bmas['14.1'] or bmas['14.2'] or bmas['14.3'])
                and bmas['11.3'] and not esf['CO01']))


@rule
def co06(bmas, esf):
    check_fields(['8'], bmas, esf)
    return bmas['8']


@rule
def co07(bmas, esf):
    check_fields(['10'], bmas, esf)
    return bmas['10']


@rule
def co08(bmas, esf):
    check_fields(['10', 'CO01', 'CO04'], bmas, esf)
    return bmas['10'] and (esf['CO01'] or esf['CO04'])


@rule
def co09(bmas, esf):
    check_fields(['15.1',
                  '15.2',
                  '15.3',
                  '15.4',
                  '15.7a',
                  '16.1',
                  '16.3',
                  '16.4'], bmas, esf)
    return ((bmas['15.1'] or bmas['15.2'] or bmas['15.3']
             or bmas['15.4'] or bmas['15.7a'])
            and not bmas['16.1'] and not bmas['16.2']
            and not bmas['16.3'] and not bmas['16.4'])


@rule
def co10(bmas, esf):
    """ Original (15.5 ja oder 15.6 ja oder 15.6a ja oder 15.6b ja oder
    16.1 ja) und 16.2 nein und 16.3 nein und 16.4 nein 15.6 makes no sense
    in this context, so it is left out
    """
    check_fields(['15.5',
                  '15.6a',
                  '15.6b',
                  '16.1',
                  '16.2',
                  '16.3',
                  '16.4'], bmas, esf)
    return ((bmas['15.5'] or bmas['15.6a'] or bmas['15.6b']
             or bmas['16.1'])
            and (not bmas['16.2'] and not bmas['16.3'] and not bmas['16.4']))


@rule
def co11(bmas, esf):
    check_fields(['16.2', '16.3', '16.4'], bmas, esf)
    return bmas['16.2'] or bmas['16.3'] or bmas['16.4']


@rule
def co15(bmas, esf):
    # Rule after Indikatorenueberleitung-CO15-CO18-2.pdf
    check_for_keine_angabe(bmas)
    if ('20' in bmas and bmas['20'] and '21' in bmas and not bmas['21'] and
     '22' in bmas and not bmas['22']):
        return False
    return True

def check_for_keine_angabe(bmas):
    if ('20' not in bmas and '21' in bmas and not bmas['21'] and '22' in bmas
        and not bmas['22']):
        raise KeyError('co15')
    elif ('20' not in bmas and '21' in bmas and not bmas['21'] and '22' not in
          bmas):
        raise KeyError('co15')
    elif ('20' not in bmas and '21' not in bmas and '22' in bmas and
          not bmas['22']):
        raise KeyError('co15')
    elif ('20' not in bmas and '21' not in bmas and '22' not in bmas):
        raise KeyError('co15')
    elif ('20' in bmas and bmas['20'] and '21' in bmas and not bmas['21'] and
          '22' not in bmas):
        raise KeyError('co15')
    elif ('20' in bmas and bmas['20'] and '21' not in bmas and '22' in bmas and
          not bmas['22']):
        raise KeyError('co15')
    elif ('20' in bmas and bmas['20'] and '21' not in bmas and
          '22' not in bmas):
        raise KeyError('co15')

@rule
def co16(bmas, esf):
    check_fields(['23'], bmas, esf)
    return bmas['23']


@rule
def co17(bmas, esf):
    if bmas.get('15.7b'):
        return True
    return bmas['25'] or bmas['15.7b']


@rule
def co18(bmas, esf):
    check_fields(['24'], bmas, esf)
    return not bmas['24']


@rule
def cr01(bmas, esf):
    """According to Issue 849 additionally applies:
    If CR04 is True, CR01 is always false
    Requires values of CO01 and 29
    and rule of CR04 (esf['CO01'] or esf['CO03']) and bmas['29']
    """
    check_fields(['CO01', '29', 'CO03', '26', '13'], bmas, esf)
    if (esf['CO01'] or esf['CO03']) and bmas['29']:
        return False
    return esf['CO03'] and bmas['26'] and not bmas['13']


@rule
def cr02(bmas, esf):
    '''
    CR02 indicator


    Rules changed, see waskiq/issue770
    '''
    check_fields(['14.5', '14.6', '14.7', '14.8', '27'], bmas, esf)
    return (not bmas['14.5']
            and not bmas['14.6']
            and not bmas['14.7']
            and not bmas['14.8']) and bmas['27']


@rule
def cr03(bmas, esf):
    return bmas['28']


@rule
def cr04(bmas, esf):
    check_fields(['CO01', 'CO03', '29'], bmas, esf)
    return (esf['CO01'] or esf['CO03']) and bmas['29']


@rule
def cr05(bmas, esf):
    '''
    Note: not all fields are checked in check_fields because CO15 till
    CO17 can be none but this indicator should still be available in that
    case. This is also the reason why these fields are retrieved with
    esf.get(),
    since we want to avoid errors if they are not set in the esf dictionary.

    See waskiq/issue2062
    '''
    check_fields(['CR01',
                  'CR02',
                  'CR03',
                  'CR04'], bmas, esf)
    return (esf['CR01']
            or esf['CR02']
            or esf['CR03']
            or esf['CR04']) \
        and \
           (esf.get('CO15')
            or esf.get('CO16')
            or esf.get('CO17'))


@rule
def po04(bmas, esf):
    check_fields(['9'], bmas, esf)
    return bmas['9']


@rule
def a2_1(bmas, esf):
    check_fields(['CR01', 'CR02', 'CR03', 'CR04'], bmas, esf)
    result = esf['CR01'] or esf['CR02'] or esf['CR03'] or esf['CR04']

    if '34' in bmas:
        return bmas['34'] or result
    return result


@rule
def a3_1(bmas, esf):
    check_fields(['33'], bmas, esf)
    return bmas['33']


@rule
def b1_1(bmas, esf):
    check_fields(['46'], bmas, esf)
    return bmas['46']


@rule
def b2_1(bmas, esf):
    check_fields(['CR02', 'CR04', '32'], bmas, esf)
    return esf['CR02'] or esf['CR04'] or bmas['32']


@rule
def b2_2(bmas, esf):
    check_fields(['37', '38', '39'], bmas, esf)
    return bmas['37'] or bmas['38'] or bmas['39']

@rule
def b3_1(bmas, esf):
    check_fields(['40', '41', '42', '43', '44', '45'], bmas, esf)
    return bmas['40'] or bmas['41'] or bmas['42'] \
           or bmas['43'] or bmas['44'] or bmas['45']

@rule
def b3_2(bmas, esf):
    check_fields(['CR02', 'CR04'], bmas, esf)
    return esf['CR02'] or esf['CR04']

@rule
def b3_3(bmas, esf):
    check_fields(['CR04'], bmas, esf)
    return esf['CR04']

@rule
def c1_1(bmas, esf):
    check_fields(['35'], bmas, esf)
    return bmas['35']

@rule
def c1_2(bmas, esf):
    check_fields(['36'], bmas, esf)
    return bmas['36']

@rule
def c6_1(bmas, esf):
    check_fields(['47'], bmas, esf)
    return bmas['47']


@rule
def po02(bmas, esf):
    check_fields(['7'], bmas, esf)
    return bmas['7'] == 'w'


if __name__ == "__main__":
    """
    Command line version: 
    Set each named key true or false and perform mapping.
    - Set True/False with +/- (default is true):
        12.1- : Set 12.1 False
        12.1+ : Set 12.1 True
        12.1  : Set 12.1 True
    - Set ranges:
        1..4+       : Set 1, 2, 3, 4 to True
        12.1..12.4- : Set 12.1, 12.2, 12.3, 12.4 to False  
    """

    form_input = {}
    # parse the command line
    import getopt
    import sys
    opts, args = getopt.getopt(sys.argv[1:], "")
    for arg in args:
        if arg[-1:] == '-':
            value = False
            arg = arg[:-1]
        elif arg[-1:] == '+':
            value = True
            arg = arg[:-1]
        else:
            value = True
        range_ = (arg.split('..'))
        if len(range_) == 2:
            range_start = map(int,range_[0].split('.'))
            range_end = map(int,range_[1].split('.'))
            if len(range_start) == len(range_end) == 2:
                for i in range(range_start[1],range_end[1]+1):
                    form_input['%d.%d' % (range_start[0],i)] = value
            else:
                for i in range(range_start[0],range_end[0]+1):
                    form_input[str(i)] = value
        else:
            form_input[arg] = value
    print form_input
    res = to_esf(bmas=form_input, esf={})
    print(res.esf)
