# -*- coding: utf-8 -*-

# This file is part of EUhu.
#
# EUhu is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# EUhu is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
#
# Author: Ludwig Reiter <ludwig.reiter@intevation.de>
#
# Rules for validating esf entries.
# Quotes are from "Practical guidance September 2015 cl.pdf"
# and from "150918Mapping_zu_Fragenkatalog_DATES_II_ueberarbeitet.xlsx"


def _check_dependencies(list1, dict1):
    for ind in list1:
        if dict1.get(ind) is None:
            return False
    return True


def rule_CO01(bmas_dict, esf_dict):
    """EMPLOYMENT STATUS:
    All individuals must be recorded in one of the employment status
    "employed", unemployed" or "inactive".
    """
    if not _check_dependencies(("CO01", "CO03", "CO05"), esf_dict):
        return None
    number_checked = 0
    for ind in ("CO01", "CO03", "CO05"):
        if esf_dict[ind]:
            number_checked += 1
    return number_checked == 1


def rule_CO02(bmas_dict, esf_dict):
    """Rule under CO02 from Mapping file."""
    if not _check_dependencies(("CO02", "CO01"), esf_dict):
        return None
    return esf_dict["CO01"] or not esf_dict["CO02"]


def rule_CO03(bmas_dict, esf_dict):
    """Rule under CO03 from Mapping file, same as rule_CO01."""
    return rule_CO01(bmas_dict, esf_dict)


def rule_CO04(bmas_dict, esf_dict):
    """Rule under CO04 from MApping file."""
    if not _check_dependencies(("CO03", "CO04"), esf_dict):
        return None
    return esf_dict["CO03"] or not esf_dict["CO04"]


def rule_CO05(bmas_dict, esf_dict):
    """Rule under CO05 from Mapping file, same as rule_CO01."""
    return rule_CO01(bmas_dict, esf_dict)


def rule_CO06(bmas_dict, esf_dict):
    """No individual can be both "below 25 years" and "above 54 years"."""
    if not _check_dependencies(("CO06", "CO07"), esf_dict):
        return None
    return not (esf_dict["CO06"] and esf_dict["CO07"])


def rule_CO07(bmas_dict, esf_dict):
    """Rule under CO07 from Mapping file, same as rule_CO06."""
    return rule_CO06(bmas_dict, esf_dict)


def rule_CO09(bmas_dict, esf_dict):
    """Rule under CO09 from Mapping file."""
    if not _check_dependencies(("CO11", "CO10"), esf_dict):
        return None
    if not _check_dependencies(("16.1", "16.2", "16.3", "16.4"), bmas_dict):
        return None
    return esf_dict["CO11"] or esf_dict["CO10"] or (
        not (bmas_dict["16.1"] or bmas_dict["16.2"] or bmas_dict["16.3"]
             or bmas_dict["16.4"]))


def rule_CO10(bmas_dict, esf_dict):
    """Rule under CO10 from Mapping file."""
    if not _check_dependencies(("CO11",), esf_dict):
        return None
    if not _check_dependencies(("16.2", "16.3", "16.4"), bmas_dict):
        return None
    return esf_dict["CO11"] or not (bmas_dict["16.2"] or bmas_dict["16.3"]
                                    or bmas_dict["16.4"])


def rule_CO11(bmas_dict, esf_dict):
    """Special Rule for CO11, see issue 717 for details or Mapping file."""
    if not _check_dependencies(("CO09", "CO10", "CO11"), esf_dict):
        return None
    number_checked = 0
    for ind in ("CO09", "CO10", "CO11"):
        if esf_dict[ind]:
            number_checked += 1
    if number_checked == 1:
        return True
    if number_checked == 0:
        return esf_dict.get("CO17")
    return False


def rule_PWE_GENDER(bmas_dict, esf_dict):
    """Special rule for PWE. Male participants are not eligible"""
    return esf_dict["PO02"]


def validate_rules(bmas_dict, esf_dict, programm=None):
    """Collect all rules"""
    res = {}
    okay = True
    rules = [("CO01", rule_CO01),
             ("CO02", rule_CO02),
             ("CO03", rule_CO03),
             ("CO04", rule_CO04),
             ("CO05", rule_CO05),
             ("CO06", rule_CO06),
             ("CO07", rule_CO07),
             ("CO09", rule_CO09),
             ("CO10", rule_CO10),
             ("CO11", rule_CO11)]

    if programm == "pwe":
        rules.append(("PWE_GENDER", rule_PWE_GENDER))

    for name, rule_foo in rules:
        res[name] = rule_foo(bmas_dict, esf_dict)

        if not res[name]:
            okay = False
    res["OKAY"] = okay

    return res
