# -*- coding: utf-8 -*-

# This file is part of EUhu.
#
# EUhu is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# EUhu is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
#
# Authors: Ludwig Reiter <ludwig.reiter@intevation.de>
#

import datetime

from euhu.esf_rules import validate_rules
from euhu.validationresult import ValidationResult
from euhu.esf import to_esf
from euhu.programs import (
    handle_single_field_transfer,
    handle_gender,
    handle_birthday,
    handle_nr_field,
    handle_lm_employment_category_employed,
    handle_lm_employment_supportpurchased,
    handle_edu_attainment_vocational,
    handle_date
)


def to_bmas(data):
    '''
    Convert input data to BMAS format
    TODO handle the not equal to WASKIQ fields 12, 15, 22, 30

    NOTE: this function is imported and used in SIB, make sure any changes here
          are also meaningful in the other project.
    '''
    res = {}
    res.update(handle_gender(data))
    res.update(handle_birthday(data))
    res.update(handle_lm_employment_supportpurchased(data))
    res.update(handle_lm_employment_category_employed(data))
    res.update(handle_pwe_lm_employment_category_unemployed(data))
    res.update(handle_single_field_transfer(data,
                                            "lm_employment_category_lookingforwork", "13"))
    res.update(handle_edu_attainment_vocational(data))

    for fn, bnr in (
            ("misc_german_nationality", "20"),
            ("da_minority", "21"),
            ("da_migrant", "22"),
            ("da_disabled", "23"),
            ("da_resident", "24"),
            ("da_other_disadvantage", "25"),
            ("new_job_search", "26"),
            ("school_job_education", "27"),
            ("new_qualification", "28"),
            ("new_job_or_self_employed", "29"),
            ("status_end_measure", "30"),
            ("minijob_to_si", "34")):
        res.update(handle_single_field_transfer(data, fn, bnr))

    res.update(handle_date(data, "end_date", "31"))
    return res


def handle_pwe_lm_employment_category_unemployed(data):
    bmas = {}
    if (data.get("lm_employment_category_unemployed") is not None):
        if str(data.get("lm_employment_category_unemployed")) == "0":
            return init_bmas()
    if (str(data.get("lm_employment_category_unemployed")) == "1") and (
                data.get("lm_employment_category_unemployement_duration") is not
                None):
        bmas = init_bmas()
        duration = int(data.get("lm_employment_category_unemployement"
                                "_duration"))
        if duration < 6:
            bmas["12.1"] = True
        elif duration < 12:
            bmas["12.2"] = True
        else:
            bmas["12.3"] = True
            val_lecut = str(data.get("lm_employment_category_"
                                     "unemployment_type"))
            if val_lecut == "1":
                bmas["12.4"] = True
            elif val_lecut == "0":
                bmas["12.4"] = False
            else:
                del bmas["12.4"]
    return bmas


def init_bmas():
    return {"12.1": False, "12.2": False, "12.3": False, "12.4": False}


def handle_pwe_edu_attaiment_school(data):
    bmas = {}
    for entity, bmas_number, entity_value in (
            ("edu_attainment_school", "15.1", "1"),
            ("edu_attainment_school", "15.2", "2"),
            ("edu_attainment_school", "15.3", "3"),
            ("edu_attainment_school", "15.4", "4"),
            ("edu_attainment_school", "15.5", "5"),
            ("edu_attainment_school", "15.6a", "6"),
            ("edu_attainment_school", "15.6b", "7"),
            ("edu_attainment_school", "15.7b", "8")):
        bmas.update(handle_nr_field(data, entity, bmas_number,
                                    entity_value))
    # See issue784 for more details. Value 10 must also map to
    # 15.7a.
    # See issue784 for more details. Value 10 must also map to
    # 15.7a.
    if data.get(entity) is not None:
        if str(data.get(entity)) in ["0", "9"]:
            bmas["15.7a"] = True
        else:
            bmas["15.7a"] = False
    return bmas


def handle_pwe_lm_employment_supportpurchased(program):
    bmas = {}
    if program.get("lm_employment_supportpurchased") is not None:
        bmas = {"11.1": False, "11.2": False, "11.3": False}
        value = str(program.get("lm_employment_supportpurchased"))
        if value == "0":
            bmas["11.1"] = False
        elif value == "1":
            bmas["11.2"] = True
        elif value == "2":
            bmas["11.3"] = True
        elif value == "3":
            bmas["11.2"] = True
            bmas["11.3"] = True
    return bmas


def validate(values):
    """Will return True if the submitted values meet the requirements
    for an evaluation of the EU output und result indicators. Else False.

    The function will first convert the PWE specific values into the
    BMAS format. The BMAS values will be translated into the EU formar.
    The EU format will be validated.

    :values: Dictionary of values to be validated.
    :returns: True or False

    """
    bmas = to_bmas(values)
    eu = to_esf(bmas, {})
    rules = validate_rules(bmas, eu.esf)
    return ValidationResult(rules,eu)
