# -*- coding: utf-8 -*-

# This file is part of EUhu.
#
# EUhu is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# EUhu is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License

from euhu.validationresult import ValidationResult
from euhu.esf import to_esf
from euhu.esf_rules import validate_rules
from euhu.programs import (
    handle_single_field_transfer,
    handle_gender,
    handle_birthday,
    handle_lm_employment_category_employed,
    handle_edu_attainment_vocational,
    handle_nr_field
)


def to_bmas(data):
    '''
    Convert EC input data to BMAS format.
    '''
    # The following BMAS indicators are preset 
    # as they are not collected by the forms
    indicators = ['11.1','11.2', '11.3', '12.1','12.2', '12.3', '12.4','14.4',
                  '14.5','14.6','14.7','14.8','15.7', '15.7a', '15.7b']
    bmas = dict.fromkeys(indicators, False)

    bmas.update(handle_gender(data))
    bmas.update(handle_birthday(data))
    bmas.update(handle_lm_employment_category_employed(data))

    # process bmas 14.9 separately
    if "14.9" in bmas: del bmas["14.9"]
    bmas.update(handle_lm_employment_category_inactive(data))

    bmas.update(handle_edu_attainment_school(data))
    bmas.update(handle_single_field_transfer(
        data, "lm_employment_category_lookingforwork", "13"))
    bmas.update(handle_edu_attainment_vocational(data))
    for field_number, bmas_number in (
            ("hh_dependentchildren", "17"),
            ("hh_singleparent_depended", "18"),
            ("hh_employed", "19"),
            ("born_germany", "20"),
            ("da_minority", "21"),
            ("da_migrant", "22"),
            ("da_disabled", "23"),
            ("da_resident", "24"),
            ("da_other_disadvantage", "25"),
            ("new_job_search", "26"),
            ("school_job_education", "27"),
            ("new_qualification", "28"),
            ("new_job_or_self_employed", "29"),
            ("status_end_measure", "30")):
        bmas.update(handle_single_field_transfer(
            data, field_number, bmas_number))
    bmas.update(handle_EC1(data))
    bmas.update(handle_ec_new_qualification_ec(data))
    return bmas


def handle_edu_attainment_school(data):
    bmas = {}
    for entity, bmas_number, entity_value in (
            ("edu_attainment_school", "15.1", "2"),
            ("edu_attainment_school", "15.2", "3"),
            ("edu_attainment_school", "15.3", "4"),
            ("edu_attainment_school", "15.4", "5"),
            ("edu_attainment_school", "15.5", "6"),
            ("edu_attainment_school", "15.6a", "7"),
            ("edu_attainment_school", "15.6b", "8")):
        bmas.update(handle_nr_field(data, entity, bmas_number,
                                    entity_value))
    return bmas

def handle_lm_employment_category_inactive(data):
    """
    Issue2368 EC: Mapping 3.4 und 3.4a -> 14.9 (Nichterwerbstätige)
    - 3.4    in Elternzeit
    - 3.4 a  erwerbstätig in Elternzeit (bis zu 30 h/Woche)
      (3.4a ist wählbar, wenn 3.4 = [Ja]
    Mapping:
    Falls 3.4a
    = [Nein], dann zählt die Person als nichterwerbstätig; 14.9 = [Ja]
    = [Ja], dann zählt die Person NICHT als nichterwerbstätig; 14.9 = [Nein]
    = [NULL] (weil 3.4 = [Nein]), dann zählt die Person NICHT als 
             nichterwerbstätig; 14.9 = [Nein]
    """
    bmas = {}
    if "lm_employment_category_inactive" in data:
        if str(data["lm_employment_category_inactive"]) == "0":
            bmas["14.9"] = False
        else:
            if str(data.get("lm_employment_category_inactive_employed")) == "0":
               bmas["14.9"] = True
            elif str(data.get("lm_employment_category_inactive_employed")) == "1":
               bmas["14.9"] = False

        # else: 3.4 = [Ja] und 3.4a = [NULL]
        # --> 14.9 = [NULL], unvollständig   
    return bmas
    

def handle_EC1(data):
    """Handle program specific sachbericht indicator EC1. See issue2187."""
    bmas = {}
    if "new_qualification" in data and "ec_new_qualification_ec" in data:
        bmas["EC1"] = str(data["new_qualification"]) == "1" and \
                      str(data["ec_new_qualification_ec"]) == "1"
    return bmas


def handle_ec_new_qualification_ec(data):
    """Handles special case for C1.2
    Works as specied under 
    
    See Issue: https://support.intevation.de/waskiq/issue3292
    
    Args:
        data (dictionary): Formdata
    """

    if check_for_yes(data, "ec_new_qualification_ec"):
        return {"36": True}
    elif check_for_no(data, "new_qualification") or \
            check_for_no(data, "ec_new_qualification_ec"):
        return {"36": False}
    else:
        return {}


def check_for_yes(data, indicator):
    return indicator in data and str(data[indicator]) == "1"


def check_for_no(data, indicator):
    return indicator in data and str(data[indicator]) == "0"


def validate(values):
    """Will return True if the submitted values meet the requirements
    for an evaluation of the EU output und result indicators. Else False.

    The function will first convert the WASKIQ specific values into the
    BMAS format. The BMAS values will be translated into the EU formar.
    The EU format will be validated.

    :values: Dictionary of values to be validated.
    :returns: True or False

    """
    bmas = to_bmas(values)
    eu = to_esf(bmas, {})
    rules = validate_rules(bmas, eu.esf)
    return ValidationResult(rules, eu)
