# -*- coding: utf-8 -*-

# This file is part of EUhu.
#
# EUhu is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# EUhu is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
#
# Authors: Ludwig Reiter <ludwig.reiter@intevation.de>
#

from euhu.programs.base import (
    to_bmas as pwe_to_bmas,
    handle_pwe_edu_attaiment_school
)
from euhu.validationresult import ValidationResult
from euhu.esf import to_esf
from euhu.esf_rules import validate_rules
from euhu.programs import handle_single_field_transfer

def to_bmas(data):
    '''
    Convert PWE input data to BMAS format.
    '''
    pwe = pwe_to_bmas(data)
    pwe.update(handle_pwe_edu_attaiment_school(data))
    pwe.update(handle_single_field_transfer(data, "minijob_to_si", "34"))
    pwe.update(handle_single_field_transfer(data, "qualification_pwe", "28"))
    ## Overwrite handling of lm_employment_category_selfemployed (issue2148)
    pwe.update(handle_lm_employment_category_selfemployed(data))
    pwe.update(handle_PWE1(pwe))
    return pwe


def handle_lm_employment_category_selfemployed(data):
    res = handle_single_field_transfer(data,
                                       "lm_employment_category_selfemployed",
                                       "14.3")
    if res.get("14.3"):
        if str(data.get("lm_employment_category_selfemployed_reduced")) == "1":
            res["14.3"] = False
            res["14.2"] = True
    return res


def handle_PWE1(pre_res):
    bmas = {}
    if pre_res.get("26") is not None or pre_res.get("27") is not None or \
       pre_res.get("28") is not None or pre_res.get("28") is not None or \
       pre_res.get("34") is not None:
        bmas["PWE1"] = pre_res.get("26") or pre_res.get("27") or \
                       pre_res.get("28") or pre_res.get("29") or \
                       pre_res.get("34")
    return bmas

def validate(values):
    """Will return True if the submitted values meet the requirements
    for an evaluation of the EU output und result indicators. Else False.

    The function will first convert the WASKIQ specific values into the
    BMAS format. The BMAS values will be translated into the EU formar.
    The EU format will be validated.

    :values: Dictionary of values to be validated.
    :returns: True or False

    """
    bmas = to_bmas(values)
    eu = to_esf(bmas, {})
    rules = validate_rules(bmas, eu.esf, "pwe")
    return ValidationResult(rules, eu)
