# -*- coding: utf-8 -*-

# This file is part of EUhu.
#
# EUhu is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# EUhu is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License

from euhu.programs.base import to_bmas as qe_to_bmas
from euhu.programs.sib import handle_sib_edu_attaiment_school
from euhu.validationresult import ValidationResult
from euhu.esf import to_esf
from euhu.esf_rules import validate_rules


def to_bmas(data):
    '''
    Convert QE input data to BMAS format.
    '''
    qe = qe_to_bmas(data)
    # QE uses the same value schema as sib in this field. Issue1382.
    qe.update(handle_sib_edu_attaiment_school(data))
    qe.update(handle_new_qualification_type(data))
    qe.update(handle_QE1(data))
    return qe


def handle_new_qualification_type(qe):
    bmas = {}
    field = "new_qualification_type"
    if field in qe:
        if str(qe.get(field)) == "0":
            bmas['35'] = True
        else:
            bmas['35'] = False
    return bmas


def handle_QE1(qe):
    bmas = {}
    if "new_qualification" in qe and "new_qualification_type" in qe:
        bmas['QE1'] = str(qe["new_qualification"]) == "1" and \
            str(qe["new_qualification_type"]) == "0"
    return bmas


def validate(values):
    """Will return True if the submitted values meet the requirements
    for an evaluation of the EU output und result indicators. Else False.

    The function will first convert the WASKIQ specific values into the
    BMAS format. The BMAS values will be translated into the EU formar.
    The EU format will be validated.

    :values: Dictionary of values to be validated.
    :returns: True or False

    """
    bmas = to_bmas(values)
    eu = to_esf(bmas, {})
    rules = validate_rules(bmas, eu.esf)
    # Due to changes regarding the working status of participants
    # all participants have a status of CO05, if they had any valid
    # status at all.
    # Details see: https://support.intevation.de/waskiq/issue3436
    if eu.esf.get('CO01') or eu.esf.get('CO03') or eu.esf.get('CO05'):
        eu.esf['CO01'] = False
        eu.esf['CO02'] = False
        eu.esf['CO03'] = False
        eu.esf['CO04'] = False
        eu.esf['CO05'] = True
        eu.esf['CO08'] = False
        if 'CR01' in eu.esf:
            eu.esf['CR01'] = False
        if 'CR04' in eu.esf:
            eu.esf['CR04'] = False
        try:
            eu.esf['CR05'] = (eu.esf['CR01']
                              or eu.esf['CR02']
                              or eu.esf['CR03']
                              or eu.esf['CR04']) \
                and \
                (eu.esf.get('CO15')
                 or eu.esf.get('CO16')
                 or eu.esf.get('CO17'))
        except:
            pass
    return ValidationResult(rules, eu)
