# -*- coding: utf-8 -*-

# This file is part of EUhu.
#
# EUhu is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# EUhu is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# Authors: Ludwig Reiter <ludwig.reiter@intevation.de>
#

from euhu.validationresult import ValidationResult
from euhu.esf import to_esf
from euhu.esf_rules import validate_rules
from euhu.programs import (
    handle_single_field_transfer,
    handle_gender,
    handle_birthday,
    handle_nr_field,
    handle_lm_employment_supportpurchased,
    handle_lm_employment_category_employed,
    handle_edu_attainment_vocational,
    handle_date
)


def to_bmas(waskiq_dict):
    '''
    Convert WASKIQ input data to BMAS format
    '''
    bmas = {}
    bmas.update(handle_gender(waskiq_dict))
    bmas.update(handle_birthday(waskiq_dict))
    bmas.update(handle_lm_employment_supportpurchased(waskiq_dict))
    bmas.update(handle_lm_employment_category_unemployed(waskiq_dict))
    bmas.update(handle_lm_employment_category_employed(waskiq_dict))
    bmas.update(handle_edu_attaiment_school(waskiq_dict))
    bmas.update(handle_single_field_transfer(waskiq_dict,
                "lm_employment_category_lookingforwork", "13"))
    bmas.update(handle_edu_attainment_vocational(waskiq_dict))
    bmas.update(handle_german_nationality(waskiq_dict))
    bmas.update(handle_migration_background(waskiq_dict))
    for entity, bmas_number in (
            ("hh_dependentchildren", "17"),
            ("hh_singleparent_depended", "18"),
            ("hh_employed", "19"),
            ("da_minority", "21"),
            ("da_disabled", "23"),
            ("da_resident", "24"),
            ("da_other_disadvantage", "25"),
            ("new_job_search", "26"),
            ("school_job_education", "27"),
            ("new_qualification", "28"),
            ("new_job_or_self_employed", "29"),
            ("status_end_measure", "30")):
        bmas.update(handle_single_field_transfer(waskiq_dict, entity, bmas_number))
    bmas.update(handle_date(waskiq_dict, "end_date", "31"))
    return bmas


def handle_migration_background(waskiq):
    bmas = {}
    field_birth = 'misc_birthcountry_germany'  # 17
    field_parents =  'misc_birthcountry_parent_germany'  # 18

    if field_birth in waskiq or field_parents in waskiq:
        if str(waskiq.get(field_birth)) == "1" or str(waskiq.get(field_parents)) == "1":
            bmas['22'] = True
        elif str(waskiq.get(field_birth)) == "0" and str(waskiq.get(field_parents)) == "0":
            bmas['22'] = False
    return bmas

def handle_german_nationality(waskiq):
    bmas = {}
    field = "misc_german_nationality"
    if field in waskiq:
        if str(waskiq.get(field)) in ("1", "2"):
            bmas['20'] = True
        elif waskiq.get(field) is not None:
            bmas['20'] = False
    return bmas


def handle_lm_employment_category_unemployed(waskiq):
    bmas = {}
    if waskiq.get("lm_employment_category_unemployed") is not None:
        bmas = {"12.1": False, "12.2": False, "12.3": False, "12.4": False}
        val_lecu = str(waskiq.get("lm_employment_category_unemployed"))
        if (val_lecu == "1"):
            bmas["12.1"] = True
        elif val_lecu == "2":
            bmas["12.2"] = True
        elif val_lecu == "3":
            bmas["12.3"] = True
            val_lecut = str(waskiq.get("lm_employment_category_"
                                       "unemployement_type"))
            if val_lecut == "1":
                bmas["12.4"] = True
            elif val_lecut == "0":
                bmas["12.4"] = False
            else:
                del bmas["12.4"]
    return bmas


def handle_edu_attaiment_school(waskiq):
    bmas = {}
    for entity, bmas_number, entity_value in (
            ("edu_attainment_school", "15.1", "1"),
            ("edu_attainment_school", "15.2", "2"),
            ("edu_attainment_school", "15.3", "3"),
            ("edu_attainment_school", "15.4", "4"),
            ("edu_attainment_school", "15.5", "5"),
            ("edu_attainment_school", "15.6a", "6"),
            ("edu_attainment_school", "15.6b", "7"),
            ("edu_attainment_school", "15.7b", "9")):
        bmas.update(handle_nr_field(waskiq, entity, bmas_number, entity_value))
    # See issue784 for more details. Value 10 must also map to
    # 15.7a.
    if waskiq.get(entity) is not None:
        if str(waskiq.get(entity)) in ["8", "10"]:
            bmas["15.7a"] = True
        else:
            bmas["15.7a"] = False
    return bmas


def validate(values):
    """Will return True if the submitted values meet the requirements
    for an evaluation of the EU output und result indicators. Else False.

    The function will first convert the WASKIQ specific values into the
    BMAS format. The BMAS values will be translated into the EU formar.
    The EU format will be validated.

    :values: Dictionary of values to be validated.
    :returns: True or False

    """
    bmas = to_bmas(values)
    eu = to_esf(bmas, {})
    rules = validate_rules(bmas, eu.esf)
    return ValidationResult(rules, eu)
